# coding: utf-8

"""
    Hatchet API

    The Hatchet API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from hatchet_sdk.clients.rest.models.v1_webhook_source_name import V1WebhookSourceName


class V1CreateWebhookRequestBase(BaseModel):
    """
    V1CreateWebhookRequestBase
    """  # noqa: E501

    source_name: V1WebhookSourceName = Field(
        description="The name of the source for this webhook", alias="sourceName"
    )
    name: StrictStr = Field(description="The name of the webhook")
    event_key_expression: StrictStr = Field(
        description="The CEL expression to use for the event key. This is used to create the event key from the webhook payload.",
        alias="eventKeyExpression",
    )
    scope_expression: Optional[StrictStr] = Field(
        default=None,
        description="The CEL expression to use for the scope. This is used to filter the correct workflow to trigger.",
        alias="scopeExpression",
    )
    static_payload: Optional[Dict[str, Any]] = Field(
        default=None,
        description="The static payload to use for the webhook. This is used to send a static payload with the webhook.",
        alias="staticPayload",
    )
    __properties: ClassVar[List[str]] = [
        "sourceName",
        "name",
        "eventKeyExpression",
        "scopeExpression",
        "staticPayload",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of V1CreateWebhookRequestBase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of V1CreateWebhookRequestBase from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "sourceName": obj.get("sourceName"),
                "name": obj.get("name"),
                "eventKeyExpression": obj.get("eventKeyExpression"),
                "scopeExpression": obj.get("scopeExpression"),
                "staticPayload": obj.get("staticPayload"),
            }
        )
        return _obj
