# coding: utf-8

"""
    Hatchet API

    The Hatchet API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Self

from hatchet_sdk.clients.rest.models.v1_log_line_level import V1LogLineLevel


class V1LogLine(BaseModel):
    """
    V1LogLine
    """  # noqa: E501

    created_at: datetime = Field(
        description="The creation date of the log line.", alias="createdAt"
    )
    message: StrictStr = Field(description="The log message.")
    metadata: Dict[str, Any] = Field(description="The log metadata.")
    retry_count: Optional[StrictInt] = Field(
        default=None, description="The retry count of the log line.", alias="retryCount"
    )
    attempt: Optional[StrictInt] = Field(
        default=None, description="The attempt number of the log line."
    )
    level: Optional[V1LogLineLevel] = Field(default=None, description="The log level.")
    __properties: ClassVar[List[str]] = [
        "createdAt",
        "message",
        "metadata",
        "retryCount",
        "attempt",
        "level",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of V1LogLine from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of V1LogLine from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "createdAt": obj.get("createdAt"),
                "message": obj.get("message"),
                "metadata": obj.get("metadata"),
                "retryCount": obj.get("retryCount"),
                "attempt": obj.get("attempt"),
                "level": obj.get("level"),
            }
        )
        return _obj
